# -*- coding: utf-8 -*-
#
#  __init__.py
#  django-hierarchy
# 
#  Created by Lars Yencken on 09-11-2008.
#  Copyright 2008 Lars Yencken. All rights reserved.
#

"""
A basic hierarchical model based on the nested set abstraction. 
"""

# vim: ts=4 sw=4 sts=4 et tw=78:
